<?php
namespace Repository;

use Util\Instance\GeneralProvider as InstanceProvider;
use Util\Utilities;
use Doctrine\ORM\EntityManager;


class Table {

    /** @var EntityManager **/
    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    /**
     * @param $conditions
     * @param $limit
     * @param $offset
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getAllTables($conditions, $limit, $offset){
        $query = <<<SQL
                SELECT * FROM (SELECT TABLE_NAME
                  FROM USER_TABLES
              ORDER BY TABLE_NAME) TAB
                WHERE 1=1  $conditions
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }

    /**
     * @param $table
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getTableObservatrion($table){
        $query = <<<SQL
                SELECT DSTABELABD FROM TABELA
                WHERE NMTABELABD = '$table'
SQL;
        return $this->entityManager->getConnection()->fetchAll($query);

    }

    /**
     * @param $column
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function gettableColumnObservations($column, $table){
        $query = <<<SQL
                SELECT DSCOLUNA FROM COLUNA
                WHERE NMCOLUNA = '$column'
                AND NMTABELABD = '$table'
                GROUP BY DSCOLUNA
SQL;
        return $this->entityManager->getConnection()->fetchAll($query);

    }

    /**
     * @param $conditions
     * @param $limit
     * @param $offset
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getAllTableColumns($conditions, $pkFilter, $limit, $offset){
        $query = <<<SQL
            SELECT *
              FROM (SELECT TABLE_DATA.*, UTC.DATA_DEFAULT
                      FROM (SELECT DISTINCT UTB.TABLE_NAME, UTB.COLUMN_NAME, UTB.DATA_TYPE, UTB.DATA_LENGTH,
                                            UTB.NULLABLE, DECODE(UCS.CONSTRAINT_TYPE, 'P', UCC.POSITION, NULL) AS PK
                              FROM USER_TAB_COLUMNS UTB
                         LEFT JOIN USER_CONS_COLUMNS UCC ON UTB.COLUMN_NAME = UCC.COLUMN_NAME
                                                        AND UTB.TABLE_NAME = UCC.TABLE_NAME
                                                        AND UCC.POSITION IS NOT NULL
                         LEFT JOIN USER_CONSTRAINTS UCS ON UCC.CONSTRAINT_NAME = UCS.CONSTRAINT_NAME) TABLE_DATA
            
                      JOIN USER_TAB_COLUMNS UTC ON TABLE_DATA.TABLE_NAME = UTC.TABLE_NAME
                                               AND TABLE_DATA.COLUMN_NAME = UTC.COLUMN_NAME)
                    WHERE 1=1
                    $pkFilter
                    $conditions
                 ORDER BY COLUMN_NAME
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }

    /**
     * @param $conditions
     * @param $limit
     * @param $offset
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getAllTableConstraints($conditions, $limit, $offset){
        $query = <<<SQL
                SELECT CONSTRAINT_NAME , CONSTRAINT_TYPE, SEARCH_CONDITION, DELETE_RULE,
                       STATUS, TO_CHAR(LAST_CHANGE, 'DD/MM/YYYY') AS LAST_CHANGE, R_CONSTRAINT_NAME
                  FROM USER_CONSTRAINTS
                 WHERE 1 = 1
                 $conditions
              ORDER BY CONSTRAINT_NAME 
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }

    /**
     * @param $conditions
     * @param $limit
     * @param $offset
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getAllTableTriggers($conditions, $limit, $offset){
        $query = <<<SQL
                SELECT TRIGGER_NAME , TRIGGER_TYPE, TRIGGERING_EVENT, 
                       DESCRIPTION, TRIGGER_BODY, STATUS
                  FROM USER_TRIGGERS
                 WHERE 1 = 1
                 $conditions
              ORDER BY TRIGGER_NAME
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }

    /**
     * @param $conditions
     * @param $limit
     * @param $offset
     * @return mixed[]
     * @throws \Doctrine\DBAL\DBALException
     */
    public function getAllTableConstraintColumns($conditions, $limit, $offset){
        $query = <<<SQL
                SELECT COLUMN_NAME, NULL AS FK
                  FROM USER_CONS_COLUMNS
                 WHERE 1 = 1
                 $conditions
              ORDER BY COLUMN_NAME 
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }

    public function getAllTableFKConstraintColumns($conditions, $limit, $offset){
        $query = <<<SQL
                SELECT * FROM (
                        SELECT UCC.CONSTRAINT_NAME ,UCC.COLUMN_NAME, UCC.TABLE_NAME,
                               RCC.COLUMN_NAME AS R_COLUMN_NAME, RCC.TABLE_NAME AS R_TABLE_NAME
                          FROM USER_CONS_COLUMNS UCC
                          JOIN USER_CONSTRAINTS  UCS ON UCS.CONSTRAINT_NAME = UCC.CONSTRAINT_NAME
                          JOIN USER_CONS_COLUMNS RCC ON RCC.CONSTRAINT_NAME = UCS.R_CONSTRAINT_NAME
                                                    AND RCC.POSITION = UCC.POSITION
                          )
                          WHERE 1=1
                          $conditions
              ORDER BY CONSTRAINT_NAME
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return Utilities::getReferenceConnection()->fetchAll($query);
    }
}